/*
 *  Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 *  Licensed under the Eclipse Public License - v 1.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *           http://www.eclipse.org/legal/epl-v10.html
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package nl.cloudfarming.client.task.shape;

import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.FutureTask;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.field.model.ActivityField;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.FieldManager;
import nl.cloudfarming.client.field.model.Shape;
import nl.cloudfarming.client.field.shape.ShapeDataObject;
import nl.cloudfarming.client.field.shape.TreatmentZoneImportCookie;
import nl.cloudfarming.client.task.field.FieldTask;
import nl.cloudfarming.client.task.field.FieldTaskManager;
import nl.cloudfarming.client.util.ProgressListener;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * <p>Callable task creator. Creates {@link List} of {@link FieldTask} 's based on the provided {@link List} of {@link TreatmentZoneImportCookie}</p>
 * 
 * <p>
 * Use like this:</br>
 *     {@link ExecutorService} executor = {@link Executors#newFixedThreadPool(1)};</br>
 *     TaskCreator creator = new TaskCreator(cookies);</br>
 *     executor.execute(creator);</br>
 *     executor.shutdown();</br>
 *     </br>
 * if you need the result or you need to wait on the execution of the callable use {@link FutureTask#get()}
 * </p>
 * @author Merijn Zengers <Merijn.Zengers@gmail.nl>
 */
@NbBundle.Messages({"TaskCreator progress handler msg=Converting shapes to tasks"})
public class TaskCreator extends FutureTask<List<FieldTask>> {

    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.task.shape.TaskCreator");

    //TODO: Create generic ImpotCookie class so that this class can create tasks for all the coockies suported by shape
    public TaskCreator(List<TreatmentZoneImportCookie> cookies) {
        super(new TaskCreatorWorker(cookies));
    }

    private static class TaskCreatorWorker implements Callable<List<FieldTask>> {

        private final List<TreatmentZoneImportCookie> cookies;
        private List<Field> fields;

        public TaskCreatorWorker(List<TreatmentZoneImportCookie> cookies) {
            this.cookies = cookies;

        }

        /**
         * Creates a list of {@link FieldTask} based on the {@link TreatmentZoneImportCookie} 's provided in the constructor
         * @return
         * @throws Exception 
         */
        @Override
        public List<FieldTask> call() throws Exception {
            List<FieldTask> fieldTasks = new ArrayList<>();
            LOGGER.log(Level.FINEST, "performing convert to task action on {0} objects", cookies.size());
            for (TreatmentZoneImportCookie cookie : cookies) {
                ShapeDataObject shapeDataObject = cookie.getShapeDataObject();
                final List<Shape> shapes = shapeDataObject.getShapes();
                final String dataObjectFileName = shapeDataObject.getPrimaryFile().getName();
                final Project project = FileOwnerQuery.getOwner(shapeDataObject.getPrimaryFile());
                final FieldManager fieldManager = Lookup.getDefault().lookup(FieldManager.class);

                //FIXME create multi process handler
                ProgressHandle progressHandle = ProgressHandleFactory.createHandle(Bundle.TaskCreator_progress_handler_msg());
                //start the progressHandle and set its limit to the number of shapes
                //contained by the shapeDataObject
                progressHandle.start(shapes.size());
                final PropertyChangeListener progressListener = new ProgressListener(
                        progressHandle, FieldManager.PROP_PROGRESS, FieldManager.PROP_FINISH);

                switch (shapeDataObject.getContentType()) {
                    //FIXME TV: Can the content type ever be fields??
                    case ShapeDataObject.CONTENT_FIELDS:
                        //if the shapeDataObject contains fields, create a FieldWorker
                        LOGGER.finest("executing field worker..");
                        fields = fieldManager.importFields(shapes, project, progressListener);
                        break;
                    case ShapeDataObject.CONTENT_TREATMENT_ZONES:
                        //if the shapeDataObject contains treatment zones, create a TreatmentZoneWorker
                        LOGGER.finest("executing treatment zone worker..");
                        fields = fieldManager.importTreatmentZones(shapes, project, dataObjectFileName, progressListener);
                        break;
                }
                fieldTasks.addAll(createTasksForFields(fields));
            }
            return fieldTasks;
        }

        /**
         * This method creates tasks for each activityField connected to a
         * field. It gets the activityFields for each field and creates a
         * FieldTask from that activityField.
         *
         * @param List of fields that contain activityFields
         */
        private List<FieldTask> createTasksForFields(List<Field> fields) {
            List<FieldTask> fieldTasks = new ArrayList<>();
            FieldTaskManager fieldTaskManager = Lookup.getDefault().lookup(FieldTaskManager.class);
            for (Field field : fields) {
                if (field.getActivityFields().isEmpty()) {
                    LOGGER.log(Level.FINEST, "Field does not contain activityFields!");
                }
                //FIXME only make task for new activity fields
                for (ActivityField activityField : field.getActivityFields()) {
                    LOGGER.log(Level.FINEST, "Creating new field task for activity field with name {0}", activityField.getName());
                    FieldTask fieldTask = new FieldTask(activityField);
                    fieldTaskManager.addFieldTask(fieldTask);
                    fieldTasks.add(fieldTask);
                }
            }
            return fieldTasks;
        }
    }
}
